import math
import ConfigParser

class CLParameters:

	##################################
	# User reconstruction parameters #
	##################################

	# Laminographic angle (deg)
	uTheta = 0 
	# Misalignment angle (deg)
	uPsi = 0 
	# Detector pixel size
	uTau = 0
	# Volume origin
	uVOx = 0
	uVOy = 0
	uVOz = 0
	# Volume size
	uVSx = 0
	uVSy = 0
	uVSz = 0 
	# Projection origin
	uPOx = 0
	uPOy = 0
	# Projection size
	uPSx = 0
	uPSy = 0
        # Projection indices
	uPIb = 0
	uPIe = 0
	# Projections name
	uPName = 'proj*.edf'
	# Volume name
	uVName = 'reco'
        # Angular range (deg)
	uPhiRange = 0
	# Extension mode
	uExt = 'brep'

	######################################
        # Internal reconstruction parameters #
	######################################

	# Laminographic angle (rad)
	iTheta = 0
	# Complementary laminographic angle
	iAlpha = 0
	# Misalignment angle (rad)
        iPsi = 0
	# Angular range (rad)
	iPhiRange = 0
	# Projection count
	iProjCnt = 0
        # Anglular step (delta phi) 
	iPhiStep = 0
        # Volume origin
        iVOx = 0
        iVOy = 0
        iVOz = 0
        # Projection origin
        iPOx = 0
	iPOy = 0
	# add for projection extension 
	iNewProjExtentAdds = [0, 0, 0, 0]
	# size of projection after extension
	iEPSz = [0, 0]
	# size of projection before fft
	iFTPSz = [0, 0]

	# Internal consts
	iExtraExt = 4 

	def __init__(self):

		# read ini file
		config = ConfigParser.RawConfigParser()
		config.read('reco.ini')

		self.uTheta = config.getfloat('Main', 'theta')
		self.uPsi   = config.getfloat('Main', 'axis-misalign')
		self.uTau   = config.getfloat('Main', 'resolution')
		self.uVOx   = config.getint('Main', 'result-origin-x')
		self.uVOy   = config.getint('Main', 'result-origin-y')
		self.uVOz   = config.getint('Main', 'result-origin-z')
		self.uVSx   = config.getint('Main', 'result-size-x')
		self.uVSy   = config.getint('Main', 'result-size-y')
		self.uVSz   = config.getint('Main', 'result-size-z')
		self.uPOx   = config.getint('Main', 'proj-origin-x')
		self.uPOy   = config.getint('Main', 'proj-origin-y')
		self.uPSx   = config.getint('Main', 'proj-size-x')
		self.uPSy   = config.getint('Main', 'proj-size-y')
		self.uPIb   = config.getint('Main', 'index-start')
		self.uPIe   = config.getint('Main', 'index-end')
		self.uPName = config.get('Main', 'projection-name')
		self.uVName = config.get('Main', 'output-edf-file')
		self.uPhiRange =  config.getfloat('Main', 'angle-range')


		# initialize internal paramteres
		self.iTheta 	= self.uTheta * math.pi / 180
		self.iPsi 	= self.uPsi * math.pi / 180
		self.iAlpha 	=  - 3 * math.pi/2 + self.iTheta;
                self.iPhiRange  = self.uPhiRange  * math.pi / 180
		self.iProjCnt 	= self.uPIe - self.uPIb + 1
		self.iPhiStep 	= self.iPhiRange / self.iProjCnt    

		self.iVOx = self.uVOx + self.uVSx/2 # dont forget del the shift in src
                self.iVOy = self.uVOy + self.uVSy/2
                self.iVOz = self.uVOz + self.uVSz/2

		self.iPOx = self.uPOx
		self.iPOy = self.uPOy
		self.iNewProjExtentAdds = self.compute_projection_adds()
		self.compute_proj_extent()
		self.resize_proj_for_fft()
		self.print_parameters()

	def compute_proj_extent(self):
		self.iEPSz[0] = self.iNewProjExtentAdds[0] +  self.uPSx + self.iNewProjExtentAdds[1]
		self.iEPSz[1] = self.iNewProjExtentAdds[2] +  self.uPSy + self.iNewProjExtentAdds[3]

	def near_powof2(self, val):
		return math.pow(2, math.ceil( math.log(val, 2)))
	
	def resize_proj_for_fft(self):
		# we make twise width for acylic convolution
		self.iFTPSz = [  self.near_powof2(2*self.iEPSz[0]), self.near_powof2(self.iEPSz[1]) ]

	def print_parameters(self):
     		print 'theta', self.iTheta
                print 'psi', self.iPsi
	        print 'alpha', self.iAlpha
                print 'range', self.iPhiRange
                print 'pcnt', self.iProjCnt
                print 'astep', self.iPhiStep
                print 'vox',  self.iVOx
                print 'voy',  self.iVOy
                print 'voz',  self.iVOz
                print 'pox', self.iPOx
		print 'poy', self.iPOy 
		print 'l r u d', self.iNewProjExtentAdds
		print 'ext - sx x sy', self.iEPSz
		print 'fft - sx x sy', self.iFTPSz
		print 'IMAGE sx ', self.uPSx
		print 'IMAGE sy ', self.uPSy
		print 'proj name', self.uPName
		print 'vout', self.uVName
		print 'vorigin',  self.uVOx,  self.uVOy,  self.uVOz
		print 'vsize',  self.uVSx,  self.uVSy,  self.uVSz


	def get_volume_corners(self):
		corners = [] 
		corners.append((           -self.iVOx,            -self.iVOy,            -self.iVOz ))
                corners.append((self.uVSx-1-self.iVOx,            -self.iVOy,            -self.iVOz ))
		corners.append((           -self.iVOx, self.uVSy-1-self.iVOy, 	    	 -self.iVOz ))
		corners.append((self.uVSx-1-self.iVOx, self.uVSy-1-self.iVOy,            -self.iVOz ))
                corners.append((           -self.iVOx,            -self.iVOy, self.uVSz-1-self.iVOz ))
                corners.append((self.uVSx-1-self.iVOx,            -self.iVOy, self.uVSz-1-self.iVOz ))
		corners.append((           -self.iVOx, self.uVSy-1-self.iVOy, self.uVSz-1-self.iVOz ))
		corners.append((self.uVSx-1-self.iVOx, self.uVSy-1-self.iVOy, self.uVSz-1-self.iVOz ))
		return corners

	def compute_transform_matrix(self, phi):
		sf = math.sin(phi);   		cf = math.cos(phi)
		ct = math.cos(self.iAlpha); 	st = math.sin(self.iAlpha) 
		cg = math.cos(self.iPsi); 	sg = math.sin(self.iPsi)

		mtrx = []
		mtrx.append( cg * cf - sg * st * sf)
		mtrx.append(-cg * sf - sg * st * cf)
		mtrx.append(-sg * ct)
		mtrx.append( sg * cf + cg * st * sf)
		mtrx.append(-sg * sf + cg * st * cf)
		mtrx.append( cg * ct)
		return mtrx

	def compute_volume_bb(self):
		corners = self.get_volume_corners()

                pnt_min = [0, 0]
		pnt_max = [0, 0]
                pnt_tmp = [0, 0]

                bInit = False 
		for pidx in range(self.uPIb, self.uPIe+1):
			ephi = self.iPhiStep * pidx
			mat = self.compute_transform_matrix(ephi)
			for cpnt in corners:
				pnt_tmp[0] = (mat[0]*cpnt[0])+(mat[1]*cpnt[1])+(mat[2]*cpnt[2])
				pnt_tmp[1] = (mat[3]*cpnt[0])+(mat[4]*cpnt[1])+(mat[5]*cpnt[2])
				if bInit is False: 
					pnt_min[0] = pnt_tmp[0]; pnt_min[1] = pnt_tmp[1]
					pnt_max[0] = pnt_tmp[0]; pnt_max[1] = pnt_tmp[1]
					bInit = True
				if ( pnt_max[0] < pnt_tmp[0] ) : pnt_max[0]=pnt_tmp[0]
				if ( pnt_max[1] < pnt_tmp[1] ) : pnt_max[1]=pnt_tmp[1]
				if ( pnt_min[0] > pnt_tmp[0] ) : pnt_min[0]=pnt_tmp[0]
				if ( pnt_min[1] > pnt_tmp[1] ) : pnt_min[1]=pnt_tmp[1]
		res = [pnt_min, pnt_max]
		return res


	def compute_projection_adds(self):
		bb = self.compute_volume_bb()

		bbl = bb[0][0]; bbr = bb[1][0]
		bbb = bb[0][1]; bbu = bb[1][1]
		pl = - self.uPOx; pr = self.uPSx - self.uPOx
		pb = - self.uPOy; pu = self.uPSy - self.uPOy 
		# print bbl, bbr, bbb, bbu
		# print pl, pr, pb, pu

		nsl = pl if (pl < bbl) else bbl
		nsr = pr if (pr > bbr) else bbr
                nsu = pu if (pu > bbu) else bbu
                nsb = pb if (pb < bbb) else bbb 
		# print nsl, nsr, nsu, nsb


		addl = math.fabs(math.floor(nsl - pl)) + self.iExtraExt
		addr = math.fabs(math.ceil (nsr - pr)) + self.iExtraExt
		addb = math.fabs(math.ceil (nsu - pu)) + self.iExtraExt
		addu = math.fabs(math.floor(nsb - pb)) + self.iExtraExt 
		# print addl, addr, addu, addb

		# init new projection origin
		self.iPOx = self.iPOx + addl
		self.iPOy = self.iPOy + addu

		return [addl, addr, addu, addb]
