from cl_parameters import * 
from gi.repository import Ufo

path = [ '/home/timurttv/_UFO-Project/git_framework3/lamino-filters-installed/lib/ufo',
         '/home/timurttv/_UFO-Project/git_framework3/ufo-filters-installed/lib/ufo' ]


# load and initialize user parameters
params =  CLParameters() 

# initialize nodes 
conf = Ufo.Configuration(paths=path)
g    = Ufo.Graph()
pm   = Ufo.PluginManager(configuration=conf)
s    = Ufo.Scheduler(configuration=conf)

reader  = pm.get_filter('reader')
reader.set_properties(path=params.uPName,  count=params.iProjCnt)

padd = pm.get_filter('padding2d')
padd.set_properties( xl = int(params.iNewProjExtentAdds[0]), 
		     xr = int(params.iFTPSz[0]-params.iNewProjExtentAdds[0]-params.uPSx), 
		     yt = int(params.iNewProjExtentAdds[2]), 
		     yb = int(params.iFTPSz[1] -params.iNewProjExtentAdds[2]-params.uPSy),   
		     mode='brep')

ramp = pm.get_filter('laminoramp')
ramp.set_properties( width  = int(params.iFTPSz[0]), 
         	     fwidth = int(params.iEPSz[0]), 
		     height = int(params.iFTPSz[1]), 
		     theta  = params.iTheta, 
		     tau    = params.uTau)

fft1 = pm.get_filter('fft')
fft1.set_properties(dimensions=2)

fft2 = pm.get_filter('fft')
fft2.set_properties(dimensions=2)

conv = pm.get_filter('laminoftconv')

ifft = pm.get_filter('ifft')
ifft.set_properties(dimensions=2)

roi = pm.get_filter('regionofinterest')
roi.set_properties( x = 0, y = 0, 
		    width  = int(params.iEPSz[0]), 
		    height = int(params.iEPSz[1]) )

rec  = pm.get_filter('laminobpgeneric')
rec.set_properties(  theta      = params.iTheta,
		     psi        = params.iPsi,
		     angle_step = params.iPhiStep,
		     vol_sx     = params.uVSx,
		     vol_sy     = params.uVSy,
		     vol_sz     = params.uVSz,
		     vol_ox     = params.iVOx,
		     vol_oy     = params.iVOy,
		     vol_oz     = params.iVOz,
		     proj_ox    = params.iPOx,
		     proj_oy    = params.iPOy)

writer = pm.get_filter('3dedfwriter')
writer.set_properties(prefix=params.uVName)

# build the computing graph

g.connect_filters(reader, padd)
g.connect_filters(padd, fft1)
g.connect_filters(ramp, fft2)

g.connect_filters_full(fft2, 0, conv, 0, Ufo.TransferMode.COPY) # filter
g.connect_filters_full(fft1, 0, conv, 1, Ufo.TransferMode.COPY) # projs

g.connect_filters(conv, ifft)
g.connect_filters(ifft, roi)
g.connect_filters(roi, rec)
g.connect_filters(rec, writer)

s.run(g)

